from ariadne import QueryType

from bot_data.abc.known_user_repository_abc import KnownUserRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_graphql.abc.filter_abc import FilterABC
from bot_graphql.filter.server_filter import ServerFilter


class Query(QueryType):
    def __init__(
        self,
        servers: ServerRepositoryABC,
        known_users: KnownUserRepositoryABC,
    ):
        QueryType.__init__(self)
        self._servers = servers
        self._known_users = known_users

        self.set_field("servers", self.resolve_servers)
        self.set_field("server_count", self.resolve_server_count)

        self.set_field("known_users", self.resolve_known_users)
        self.set_field("known_user_count", self.resolve_known_users_count)

    @FilterABC.resolve_filter_annotation
    def resolve_servers(self, *_, filter: ServerFilter = None):
        if filter is not None:
            return filter.filter(self._servers.get_servers())
        else:
            return self._servers.get_servers()

    def resolve_server_count(self, *_):
        return self._servers.get_servers().count()

    def resolve_known_users(self, *_):
        return self._known_users.get_users()

    def resolve_known_users_count(self, *_):
        return self._known_users.get_users().count()
